package service

import (
	"context"
	"log/slog"
	"time"
)
import (
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
	"google.golang.org/protobuf/types/known/emptypb"
)

import (
	"auth-server/common"
	"auth-server/logger"
	pb "auth-server/rpc_idl/dr_dcmtk_pb"
)

var DcmtkClient *Dcmtk

func init() {
	DcmtkClient = new(Dcmtk)
}

type Dcmtk struct {
	basicClient pb.BasicClient
	dcmClient   pb.DcmClient
}

func (s *Dcmtk) Setup() {
	conn, err := grpc.NewClient(common.ServerConfig.Dcmtk,
		grpc.WithTransportCredentials(insecure.NewCredentials()),
		grpc.WithUnaryInterceptor(logger.GRPCLoggerClientInterceptor(logger.WithGroup("grpc_c[dcmtk]"))),
	)
	if err != nil {
		slog.Error("NewClient failed:", err)
		panic(err)
	}
	slog.Info("dcmtk conn success", "conn", conn.GetState())

	s.basicClient = pb.NewBasicClient(conn)
	s.dcmClient = pb.NewDcmClient(conn)
}

func (s *Dcmtk) GetSoftwareInfo() *pb.SoftwareInfoReply {
	ctx, cancel := context.WithTimeout(context.Background(), 200*time.Millisecond)
	defer cancel()
	r, err := s.basicClient.SoftwareInfo(ctx, &emptypb.Empty{})
	if err != nil {
		return &pb.SoftwareInfoReply{
			Module:  "",
			Desc:    "",
			Build:   "",
			Version: "",
		}
	}
	return r
}

func (s *Dcmtk) GenerateUniqueIdentifier(ctx context.Context, flag string, number int32) (*pb.UidReply, error) {
	SiteUidRoot := "1.2.276.0.1000000.5"
	uidRoot := SiteUidRoot
	switch flag {
	case "study":
		uidRoot += ".1.2"
	case "series":
		uidRoot += ".1.3"
	case "sop":
		uidRoot += ".1.4"
	default:
		return nil, common.InvalidParam.Desc("invalid param 'flag'")
	}
	if number < 1 || number > 100 {
		return nil, common.InvalidParam.Desc("number must be between 1 and 100")
	}
	r, err := s.dcmClient.GenerateUniqueIdentifier(ctx, &pb.UidRootRequest{
		UidRoot: uidRoot,
		Number:  number,
	})
	return r, err
}
