package service

import (
	"context"
	"log/slog"
	"time"
)
import (
	"google.golang.org/grpc"
	"google.golang.org/grpc/credentials/insecure"
	"google.golang.org/protobuf/types/known/emptypb"
)

import (
	"auth-server/common"
	"auth-server/logger"
	pb "auth-server/rpc_idl/dr_resource_pb"
)

var ResourceClient *Resource

func init() {
	ResourceClient = new(Resource)
}

type Resource struct {
	basicClient  pb.BasicClient
	configClient pb.ConfigClient
}

func (s *Resource) Setup() {
	conn, err := grpc.NewClient(common.ServerConfig.Resource,
		grpc.WithTransportCredentials(insecure.NewCredentials()),
		grpc.WithUnaryInterceptor(logger.GRPCLoggerClientInterceptor(logger.WithGroup("grpc_c[resource]"))),
	)
	if err != nil {
		slog.Error("NewClient failed:", err)
		panic(err)
	}
	slog.Info("resource conn success", "conn", conn.GetState())

	s.basicClient = pb.NewBasicClient(conn)
	s.configClient = pb.NewConfigClient(conn)
}

func (s *Resource) GetSoftwareInfo() *pb.SoftwareInfoReply {
	ctx, cancel := context.WithTimeout(context.Background(), 200*time.Millisecond)
	defer cancel()
	r, err := s.basicClient.SoftwareInfo(ctx, &emptypb.Empty{})
	if err != nil {
		return &pb.SoftwareInfoReply{
			Module:  "",
			Desc:    "",
			Build:   "",
			Version: "",
		}
	}
	return r
}

func (s *Resource) GetOptions(ctx context.Context, group, flag *string) (*pb.OptionReply, error) {
	if group == nil {
		return nil, common.MissingParam.SetParam("Name", "group")
	}
	if flag == nil {
		return nil, common.MissingParam.SetParam("Name", "group")
	}
	r, err := s.configClient.GetOptions(ctx, &pb.OptionRequest{
		Group: *group,
		Flag:  *flag,
	})
	return r, err
}

func (s *Resource) GetConfigOptionList(ctx context.Context, flag string, enable bool) ([]*pb.ConfigOption, error) {
	r, err := s.configClient.ConfigOptionList(ctx, &pb.ConfigOptionListRequest{
		Flag:      flag,
		IsEnabled: enable,
	})
	if err != nil {
		return nil, err
	}
	return r.GetConfigOption(), err
}
